<?php
App::uses('AuthComponent', 'Controller/Component');
App::uses('Sanitize', 'Utility');
/**
 * Class User
 *
 * @property Role $Role
 * @property Category $Category
 * @property Article $Article
 * @property Field $Field
 * @property Comment $Comment
 * @property Page $Page
 *
 * @method findByUsername(string $username)
 */
class User extends AppModel
{
    public $name = 'User';
    public $hasMany = array(
        'Article' => array(
            'dependent' => true
        ),
        'Comment' => array(
            'dependent' => true
        ),
	    'MessageSender' => array(
		    'className' => 'Message',
		    'foreignKey' => 'sender_user_id',
		    'dependent' => true
	    ),
	    'MessageReceiver' => array(
		    'className' => 'Message',
		    'foreignKey' => 'receiver_user_id',
		    'dependent' => true
	    ),
        'Block',
        'Category',
        'Field',
        'FieldType',
        'Page',
        'File',
        'Media',
	    'ArticleRevision'
    );
    public $belongsTo = array(
        'Role'
    );

    public $validate = array(
        'username' => array(
            array(
                'rule' => 'notEmpty',
                'message' => 'Username cannot be empty'
            ),
            array(
                'rule' => 'isUnique',
                'message' => 'This username is already taken'
            )
        ),
        'email' => array(
            array(
                'rule' => 'notEmpty',
                'message' => 'Email cannot be empty'
            ),
            array(
                'rule' => 'isUnique',
                'message' => 'This email is already taken'
            )
        ),
        'password' => array(
            array(
                'rule' => 'requirePassword',
                'message' => 'Password cannot be empty'
            ),
            array(
                'rule' => array(
                    'minLength', 
                    4
                ),
                'message' => 'Must be at least 4 characters'
            ),
            array(
                'rule' => array(
                    'passCompare'
                ),
                'message' => 'The passwords do not match'
            )
        )
    );

    /**
     * @var array
     */
    public $actsAs = array(
	    'Slug' => array(
	        'field' => 'username',
	        'slugField' => 'username'
	    ),
	    'Delete'
    );

    public function passCompare() {
        return ($this->data[$this->alias]['password'] === $this->data[$this->alias]['password_confirm']);
    }

    public function requirePassword()
    {
        if (empty($this->data[$this->alias]['id']) && empty($this->data[$this->alias]['password']))
        {
            return false;
        }

        return true;
    }

    /**
    * Before Save
    *
    * @param array $options
    *
    * @return boolean
    */
    public function beforeSave($options = array())
    {
        $path = WWW_ROOT . 'uploads' . DS . 'avatars' . DS;

        $this->data = Sanitize::clean($this->data, array(
            'encode' => false,
            'remove_html' => false
        ));

        if (!empty($this->data['User']['settings']['avatar']) && 
            is_array($this->data['User']['settings']['avatar']))
        {
            if (!empty($this->data['User']['id']))
            {
                $id = $this->data['User']['id'];
            } else {
                $id = time();
            }

            $filename = $id . '_' . $this->data['User']['settings']['avatar']['name'];

            if (move_uploaded_file(
                    $this->data['User']['settings']['avatar']['tmp_name'], 
                    $path . $filename
                ))
            {
                $this->data['User']['settings']['avatar'] = $filename;
            }
            else
            {
                unset($this->data['User']['settings']['avatar']);
            }
        }

        if (!empty($this->data['User']['settings']['old_avatar']))
        {
            $file = $this->data['User']['settings']['old_avatar'];

            if (file_exists($path . $file))
            {
                unlink($path . $file);
            }
        }

        if (!empty($this->data['Security']))
        {
            $this->data['User']['security_answers'] = json_encode($this->data['Security']);
        }
        
        if (!empty($this->data['User']['settings']))
        {
            $this->data['User']['settings'] = json_encode(
                $this->data['User']['settings']
            );
        }

        if (!empty($this->data['User']['password'])) {
          $this->data['User']['password'] = AuthComponent::password(
            $this->data['User']['password']
          );
        }
        else
        {
            unset($this->data['User']['password']);
        }

        return true;
    }

	/**
	 * Get Security Answers
	 *
	 * @param $data
	 * @return array
	 */
	public function getSecurityAnswers($data)
    {
        $results = array();
        if (!empty($data['User']['security_answers'])) {
            foreach(json_decode(stripslashes($data['User']['security_answers']), true) as $key => $row)
            {
                if (!empty($row['question']) && !empty($row['answer'])) {
                    $results[$key]['question'] = str_replace("'","",$row['question']);
                    $results[$key]['answer'] = $row['answer'];
                }

                if ($key == 'activate_code')
                {
                    $results['activate_code'] = $row;
                }
                elseif ($key == 'activate_time')
                {
                    $results['activate_time'] = $row;
                }
            }
        }

        return $results;
    }

	/**
	 * Get Security Question
	 * This enables getting a random security question/answer or a specific one to match the user supplied answer.
	 *
	 * @param $data
	 * @param bool $rand
	 * @param null $id
	 * @return array
	 */
	public function getSecurityQuestion($data, $rand = true, $id = null)
	{
		$question = '';
		$question_key = '';
		$answer = '';
		if (!empty($data['User']['security_answers'])) {
			$settings = json_decode(stripslashes($data['User']['security_answers']), true);

			if (!empty($settings)) {
				$security = array();
				foreach($settings as $key => $row) {
					if (is_array($row)) {
						if ($rand || $id == $key)
							$security[$key] = $row;
					}
				}

				if (!empty($security)) {
					if ($rand) {
						reset($security);
						$first_key = key($security);

						end($security);
						$last_key = key($security);
						reset($security);

						$question_key = rand($first_key, $last_key);
					} else {
						$question_key = $id;
					}

					$question = $security[$question_key]['question'];
					$answer = $security[$question_key]['answer'];
				}
			}
		}

		return array(
			'question' => $question,
			'question_key' => $question_key,
			'answer' => $answer
		);
	}

    public function getSecurityOptions($data)
    {
        $results = array();
        if (!empty($data['SettingValue']['data_options']))
        {
            foreach($data['SettingValue']['data_options'] as $row)
            {
                $results[$row] = $row;
            }
        }

        return $results;
    }

    public function getBlockData($data, $user_id)
    {
        $cond = array(
            'conditions' => array(
                'User.status !=' => 0
            ),
            'contain' => array(
                'Role',
                'Article'
            )
        );

        if (!empty($data['limit'])) {
            $cond['limit'] = $data['limit'];
        }

        if (!empty($data['order_by'])) {
            if ($data['order_by'] == "rand") {
                $data['order_by'] = 'RAND()';
            }

            $cond['order'] = 'User.'.$data['order_by'].' '.$data['order_dir'];
        }

        if (!empty($data['data'])) {
            $cond['conditions']['User.id'] = $data['data'];
        }

        return $this->find('all', $cond);
    }

    /**
     * @param array $data
     * @param array $field
     * @return array
     */
    public function getModuleData($data = array(), $field = array())
    {
        if (empty($data) || empty($field))
        {
            return $data;
        }

        $view = new AdaptcmsView();
        $view->autoRender = false;

        $user_data = array();
        foreach($data as $key => $row)
        {
            if (!empty($row['User']))
            {
                if (empty($user_data[$row['User']['id']]))
                {
                    $value = $this->Field->ModuleValue->getValue($field, $row['User']['id'], $view);

                    $user_data[$row['User']['id']] = $value;
                    $data[$key]['User']['Data'][$field['Field']['title']] = $value;
                }
                else
                {
                    $data[$key]['User']['Data'][$field['Field']['title']] = $user_data[$row['User']['id']];
                }
            }
        }

        return $data;
    }

	/**
	 * Merge Module Data
	 *
	 * @param $request_data
	 * @param $fields
	 *
	 * @return mixed
	 */
	public function mergeModuleData($request_data, $fields)
	{
		if (!empty($fields)) {
			foreach($fields as $key => $field) {
				foreach($request_data as $row) {
					if ($row['field_id'] == $field['Field']['id']) {
						$fields[$key]['ModuleValue'][0]['data'] = $row['data'];
					}
				}
			}
		}

		return $fields;
	}
}